package router

import (
	"encoding/json"
	"net/http"
	"strings"

	"goapi-template/src/handlers"
	"goapi-template/src/storage"
)

// SetupRoutes configures and returns the HTTP router
func SetupRoutes() *http.ServeMux {
	mux := http.NewServeMux()

	// Initialize storage and handlers
	todoStorage := storage.NewTodoStorage()
	todoHandler := handlers.NewTodoHandler(todoStorage)

	// Setup todo routes
	mux.HandleFunc("/todos", func(w http.ResponseWriter, r *http.Request) {
		// Enable CORS
		w.Header().Set("Access-Control-Allow-Origin", "*")
		w.Header().Set("Access-Control-Allow-Methods", "GET, POST, PUT, DELETE, OPTIONS")
		w.Header().Set("Access-Control-Allow-Headers", "Content-Type")

		if r.Method == "OPTIONS" {
			w.WriteHeader(http.StatusOK)
			return
		}

		switch r.Method {
		case "GET":
			todoHandler.GetTodos(w, r)
		case "POST":
			todoHandler.CreateTodo(w, r)
		default:
			http.Error(w, "Method not allowed", http.StatusMethodNotAllowed)
		}
	})

	mux.HandleFunc("/todos/", func(w http.ResponseWriter, r *http.Request) {
		// Enable CORS
		w.Header().Set("Access-Control-Allow-Origin", "*")
		w.Header().Set("Access-Control-Allow-Methods", "GET, POST, PUT, DELETE, OPTIONS")
		w.Header().Set("Access-Control-Allow-Headers", "Content-Type")

		if r.Method == "OPTIONS" {
			w.WriteHeader(http.StatusOK)
			return
		}

		// Check if this is a specific todo request (has ID)
		if strings.Count(strings.Trim(r.URL.Path, "/"), "/") != 1 {
			http.Error(w, "Not found", http.StatusNotFound)
			return
		}

		switch r.Method {
		case "PUT":
			todoHandler.UpdateTodo(w, r)
		case "DELETE":
			todoHandler.DeleteTodo(w, r)
		default:
			http.Error(w, "Method not allowed", http.StatusMethodNotAllowed)
		}
	})

	// Root endpoint showing available endpoints
	mux.HandleFunc("/", func(w http.ResponseWriter, r *http.Request) {
		if r.URL.Path != "/" {
			http.Error(w, "Not found", http.StatusNotFound)
			return
		}

		w.Header().Set("Content-Type", "application/json")
		w.WriteHeader(http.StatusOK)
		response := map[string]interface{}{
			"message": "Todo API",
			"endpoints": map[string]string{
				"GET /todos":        "Get all todos",
				"POST /todos":       "Create todo",
				"PUT /todos/:id":    "Update todo",
				"DELETE /todos/:id": "Delete todo",
			},
		}
		json.NewEncoder(w).Encode(response)
	})

	return mux
}
