package storage

import (
	"errors"
	"sync"
	"time"

	"goapi-template/src/models"
)

var (
	ErrTodoNotFound = errors.New("todo not found")
)

// TodoStorage provides in-memory storage for todos
type TodoStorage struct {
	todos  map[int]*models.Todo
	nextID int
	mutex  sync.RWMutex
}

// NewTodoStorage creates a new TodoStorage instance
func NewTodoStorage() *TodoStorage {
	return &TodoStorage{
		todos:  make(map[int]*models.Todo),
		nextID: 1,
	}
}

// GetAll returns all todos
func (ts *TodoStorage) GetAll() []*models.Todo {
	ts.mutex.RLock()
	defer ts.mutex.RUnlock()

	todos := make([]*models.Todo, 0, len(ts.todos))
	for _, todo := range ts.todos {
		todos = append(todos, todo)
	}
	return todos
}

// GetByID returns a todo by ID
func (ts *TodoStorage) GetByID(id int) (*models.Todo, error) {
	ts.mutex.RLock()
	defer ts.mutex.RUnlock()

	todo, exists := ts.todos[id]
	if !exists {
		return nil, ErrTodoNotFound
	}
	return todo, nil
}

// Create creates a new todo
func (ts *TodoStorage) Create(req models.CreateTodoRequest) *models.Todo {
	ts.mutex.Lock()
	defer ts.mutex.Unlock()

	now := time.Now()
	todo := &models.Todo{
		ID:          ts.nextID,
		Title:       req.Title,
		Description: req.Description,
		Completed:   false,
		CreatedAt:   now,
		UpdatedAt:   now,
	}

	ts.todos[ts.nextID] = todo
	ts.nextID++

	return todo
}

// Update updates an existing todo
func (ts *TodoStorage) Update(id int, req models.UpdateTodoRequest) (*models.Todo, error) {
	ts.mutex.Lock()
	defer ts.mutex.Unlock()

	todo, exists := ts.todos[id]
	if !exists {
		return nil, ErrTodoNotFound
	}

	if req.Title != nil {
		todo.Title = *req.Title
	}
	if req.Description != nil {
		todo.Description = *req.Description
	}
	if req.Completed != nil {
		todo.Completed = *req.Completed
	}
	todo.UpdatedAt = time.Now()

	return todo, nil
}

// Delete deletes a todo by ID
func (ts *TodoStorage) Delete(id int) error {
	ts.mutex.Lock()
	defer ts.mutex.Unlock()

	if _, exists := ts.todos[id]; !exists {
		return ErrTodoNotFound
	}

	delete(ts.todos, id)
	return nil
}
